@@ -5,8 +5,8 @@ import traceback
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from ._version import __author__, __version__  # noqa: F401
-from .helpers import (MONITOR_MANUFACTURER_CODES,  # noqa: F401
-                      ScreenBrightnessError, flatten_list)
+from .helpers import MONITOR_MANUFACTURER_CODES  # noqa: F401
+from .helpers import ScreenBrightnessError, flatten_list, logarithmic_range
 
 
 def get_brightness(
@@ -133,6 +133,8 @@ def fade_brightness(
     interval: float = 0.01,
     increment: int = 1,
     blocking: bool = True,
+    force: bool = False,
+    logarithmic: bool = True,
     **kwargs
 ) -> Union[List[threading.Thread], List[int]]:
     '''
@@ -145,6 +147,10 @@ def fade_brightness(
         interval (float or int): the time delay between each step in brightness
         increment (int): the amount to change the brightness by per step
         blocking (bool): whether this should occur in the main thread (`True`) or a new daemonic thread (`False`)
+        force (bool): [*Linux Only*] if False the brightness will never be set lower than 1.
+            This is because on most displays a brightness of 0 will turn off the backlight.
+            If True, this check is bypassed
+        logarithmic (bool): follow a logarithmic brightness curve when adjusting the brightness
         kwargs (dict): passed directly to `set_brightness`.
             Any compatible kwargs are passed to `filter_monitors` as well. (eg: display, method...)
 
@@ -173,11 +179,13 @@ def fade_brightness(
         ```
     '''
     def fade(start, finish, increment, monitor):
-        for i in range(min(start, finish), max(start, finish), increment):
-            val = i
-            if start > finish:
-                val = start - (val - finish)
-            monitor.set_brightness(val, no_return=True)
+        range_func = logarithmic_range if logarithmic else range
+
+        if start > finish:
+            increment = -abs(increment)
+
+        for value in range_func(start, finish, increment):
+            monitor.set_brightness(value, no_return=True)
             time.sleep(interval)
 
         if monitor.get_brightness() != finish:
@@ -193,33 +201,19 @@ def fade_brightness(
                 'display', 'haystack', 'method', 'include'
             )}
         )
-    except (LookupError, ValueError) as e:
-        if not platform.system() == 'Linux':
-            raise ScreenBrightnessError(err_msg(e)) from e
-
-        method = kwargs.get('method', None)
-        if isinstance(e, LookupError) and method is None:
-            # if a user ONLY has XBacklight installed and no method was specified
-            # then filter_monitors will raise a LookupError. In this case, default to using
-            # linux.XBacklight method
-            available_monitors = [_OS_MODULE.XBacklight]
-        elif isinstance(e, ValueError) and method.lower() == 'xbacklight':
-            # low level `list_monitors_info` will raise a ValueError if 'xbacklight' is passed as the method
-            # because xbacklight cannot query individual monitor information
-            available_monitors = [_OS_MODULE.XBacklight]
-        else:
-            raise ScreenBrightnessError(err_msg(e)) from e
-
     except Exception as e:
         raise ScreenBrightnessError(err_msg(e)) from e
 
+    # minimum brightness value
+    if platform.system() == 'Linux' and not force:
+        lower_bound = 1
+    else:
+        lower_bound = 0
+
     threads = []
     for i in available_monitors:
         try:
-            if platform.system() == 'Linux' and i == _OS_MODULE.XBacklight:
-                monitor = i
-            else:
-                monitor = Monitor(i)
+            monitor = Monitor(i)
 
             # same effect as monitor.is_active()
             current = monitor.get_brightness()
@@ -234,8 +228,8 @@ def fade_brightness(
 
             st = current if st is None else st
             # make sure both values are within the correct range
-            fi = min(max(int(float(fi)), 0), 100)
-            st = min(max(int(float(st)), 0), 100)
+            fi = min(max(int(float(fi)), lower_bound), 100)
+            st = min(max(int(float(st)), lower_bound), 100)
 
             t1 = threading.Thread(target=fade, args=(st, fi, increment, monitor))
             t1.start()
@@ -452,7 +446,7 @@ class Monitor():
             if value is not None:
                 return key, value
 
-    def set_brightness(self, value: int, no_return: bool = True) -> Union[None, int]:
+    def set_brightness(self, value: int, no_return: bool = True, force: bool = False) -> Union[None, int]:
         '''
         Sets the brightness for this display. See `set_brightness` for the full docs
 
@@ -460,6 +454,9 @@ class Monitor():
             value (int): the brightness value to set the display to (from 0 to 100)
             no_return (bool): if true, this function returns `None`
                 Otherwise it returns the result of `Monitor.get_brightness`
+            force (bool): [*Linux Only*] if False the brightness will never be set lower than 1.
+                This is because on most displays a brightness of 0 will turn off the backlight.
+                If True, this check is bypassed
 
         Returns:
             None: if `no_return==True`
@@ -477,7 +474,12 @@ class Monitor():
         # refresh display info, in case another display has been unplugged or something
         # which would change the index of this display
         self.get_info()
-        value = max(0, min(value, 100))
+        # min brightness value
+        if platform.system() == 'Linux' and not force:
+            lower_bound = 1
+        else:
+            lower_bound = 0
+        value = max(lower_bound, min(value, 100))
         self.method.set_brightness(value, display=self.index)
         if no_return:
             return None
@@ -639,46 +641,62 @@ def filter_monitors(
         # EG output: [{'name': 'BenQ GL2450H', 'model': 'GL2450H', ... }]
         ```
     '''
-    # if we have been provided with a list of monitors to sift through then use that
-    # otherwise, get the info ourselves
-    if haystack:
-        monitors_with_duplicates = haystack
-        if method is not None:
-            monitors_with_duplicates = [i for i in haystack if method.lower() == i['method'].__name__.lower()]
-    else:
-        monitors_with_duplicates = list_monitors_info(method=method, allow_duplicates=True)
-
     if display is not None and type(display) not in (str, int):
         raise TypeError(f'display kwarg must be int or str, not "{type(display).__name__}"')
 
-    # This loop does two things: 1. Filters out duplicate monitors and 2. Matches the display kwarg (if applicable)
-    unique_identifiers = []
-    monitors = []
-    for monitor in monitors_with_duplicates:
-        # find a valid identifier for a monitor, excluding any which are equal to None
-        added = False
-        for identifier in ['edid', 'serial', 'name', 'model'] + include:
-            if monitor.get(identifier, None) is not None:
-                # check we haven't already added the monitor
-                if monitor[identifier] not in unique_identifiers:
-                    # check if the display kwarg (if str) matches this monitor
-                    if monitor[identifier] == display or not isinstance(display, str):
-                        # if valid and monitor[identifier] not in unique_identifiers:
-                        if not added:
-                            monitors.append(monitor)
-                            unique_identifiers.append(monitor[identifier])
-                            added = True
+    def get_monitor_list():
+        # if we have been provided with a list of monitors to sift through then use that
+        # otherwise, get the info ourselves
+        if haystack:
+            monitors_with_duplicates = haystack
+            if method is not None:
+                monitors_with_duplicates = [i for i in haystack if method.lower() == i['method'].__name__.lower()]
+        else:
+            monitors_with_duplicates = list_monitors_info(method=method, allow_duplicates=True)
 
-                        # if the display kwarg is an integer and we are currently at that index
-                        if isinstance(display, int) and len(monitors) - 1 == display:
-                            return [monitor]
-                        if added:
-                            break
-                else:
-                    # if we have already added a monitor with the same identifier
-                    # then any info matching this monitor will match the other one
-                    # so exit the checking now
-                    break
+        return monitors_with_duplicates
+
+    def filter_monitor_list():
+        # This loop does two things: 1. Filters out duplicate monitors and 2. Matches the display kwarg (if applicable)
+        unique_identifiers = []
+        monitors = []
+        for monitor in monitors_with_duplicates:
+            # find a valid identifier for a monitor, excluding any which are equal to None
+            added = False
+            for identifier in ['edid', 'serial', 'name', 'model'] + include:
+                if monitor.get(identifier, None) is not None:
+                    # check we haven't already added the monitor
+                    if monitor[identifier] not in unique_identifiers:
+                        # check if the display kwarg (if str) matches this monitor
+                        if monitor[identifier] == display or not isinstance(display, str):
+                            # if valid and monitor[identifier] not in unique_identifiers:
+                            if not added:
+                                monitors.append(monitor)
+                                unique_identifiers.append(monitor[identifier])
+                                added = True
+
+                            # if the display kwarg is an integer and we are currently at that index
+                            if isinstance(display, int) and len(monitors) - 1 == display:
+                                return [monitor]
+                            if added:
+                                break
+                    else:
+                        # if we have already added a monitor with the same identifier
+                        # then any info matching this monitor will match the other one
+                        # so exit the checking now
+                        break
+        return monitors
+
+    monitors_with_duplicates = get_monitor_list()
+    monitors = filter_monitor_list()
+    for _ in range(3):
+        if monitors == [] and not haystack:
+            # try again
+            time.sleep(0.4)
+            monitors_with_duplicates = get_monitor_list()
+            monitors = filter_monitor_list()
+        else:
+            break
 
     # if no monitors matched the query OR if display kwarg was an int
     # if the latter and we made it this far then the int was out of range
@@ -711,16 +729,6 @@ def __brightness(
 
     try:
         monitors = filter_monitors(display=display, method=method)
-    except (LookupError, ValueError) as e:
-        if platform.system() == 'Linux' and display is None and method in ('xbacklight', None):
-            try:
-                if meta_method == 'set':
-                    _OS_MODULE.XBacklight.set_brightness(*args)
-                output.append(_OS_MODULE.XBacklight.get_brightness())
-            except Exception as e:
-                format_exc('XBacklight', e)
-        else:
-            format_exc('filter_monitors', e)
     except Exception as e:
         format_exc('filter_monitors', e)
     else:
