@@ -5,13 +5,12 @@ import os
 import platform
 import subprocess
 import time
+import warnings
 from typing import List, Optional, Tuple, Union
 
 if platform.system() == 'Linux':
     import fcntl
 
-import warnings
-
 from . import filter_monitors, get_methods
 from .helpers import EDID, __cache__, _monitor_brand_lookup
 
@@ -57,42 +56,46 @@ class SysFiles:
             edp_info = sbc.linux.SysFiles.get_display_info('edp-backlight')[0]
             ```
         '''
-        displays = {}
-        index = 0
+        subsystems = set()
         for folder in os.listdir('/sys/class/backlight'):
-            if os.path.isfile(f'/sys/class/backlight/{folder}/device/edid'):
-                device = {
-                    'name': folder,
-                    'path': f'/sys/class/backlight/{folder}',
-                    'method': cls,
-                    'index': index,
-                    'model': None,
-                    'serial': None,
-                    'manufacturer': None,
-                    'manufacturer_id': None,
-                    'edid': EDID.hexdump(f'/sys/class/backlight/{folder}/device/edid'),
-                    'scale': None
-                }
+            if os.path.isdir(f'/sys/class/backlight/{folder}/subsystem'):
+                subsystems.add(tuple(os.listdir(f'/sys/class/backlight/{folder}/subsystem')))
 
+        all_displays = {}
+        index = 0
+
+        for subsystem in subsystems:
+
+            device = {
+                'name': subsystem[0],
+                'path': f'/sys/class/backlight/{subsystem[0]}',
+                'method': cls,
+                'index': index,
+                'model': None,
+                'serial': None,
+                'manufacturer': None,
+                'manufacturer_id': None,
+                'edid': None,
+                'scale': None
+            }
+
+            for folder in subsystem:
+                # subsystems like intel_backlight usually have an acpi_video0
+                # counterpart, which we don't want so lets find the 'best' candidate
                 try:
-                    # try to get resolution of brightness settings
-                    with open(os.path.join(device['path'], 'max_brightness'), 'r') as f:
-                        device['scale'] = int(f.read().rstrip('\n')) / 100
+                    with open(os.path.join(f'/sys/class/backlight/{folder}/max_brightness')) as f:
+                        scale = int(f.read().rstrip(' \n')) / 100
+
+                    # use the display with the highest resolution scale
+                    if device['scale'] is None or scale > device['scale']:
+                        device['name'] = folder
+                        device['path'] = f'/sys/class/backlight/{folder}'
+                        device['scale'] = scale
                 except (FileNotFoundError, TypeError):
-                    # if the scale cannot be figured out then exclude the display
                     continue
 
-                # now check for duplicate displays
-                if device['edid'] in displays:
-                    if (
-                        displays[device['edid']]['scale'] == 1
-                        or displays[device['edid']]['scale'] >= device['scale']
-                    ):
-                        # if there is a duplicate display, pick the display with
-                        # a scale equal to 1 or the display with the highest scale.
-                        # Reduces rounding errrors when converting percentages
-                        # to 'actual' values
-                        continue
+            if os.path.isfile('%s/device/edid' % device['path']):
+                device['edid'] = EDID.hexdump('%s/device/edid' % device['path'])
 
                 for key, value in zip(
                     ('manufacturer_id', 'manufacturer', 'model', 'name', 'serial'),
@@ -102,13 +105,13 @@ class SysFiles:
                         continue
                     device[key] = value
 
-                displays[device['edid']] = device
-                index += 1
+            all_displays[device['edid']] = device
+            index += 1
 
-        displays = list(displays.values())
+        all_displays = list(all_displays.values())
         if display is not None:
-            displays = filter_monitors(display=display, haystack=displays, include=['path'])
-        return displays
+            all_displays = filter_monitors(display=display, haystack=all_displays, include=['path'])
+        return all_displays
 
     @classmethod
     def get_brightness(cls, display: Optional[int] = None) -> List[int]:
@@ -674,75 +677,6 @@ class Light:
         return results
 
 
-class XBacklight:
-    '''
-    DEPRECATED.
-
-    A collection of screen brightness related methods using the xbacklight executable'''
-
-    executable: str = 'xbacklight'
-    '''the xbacklight executable to be called'''
-
-    @staticmethod
-    def __warn_deprecated():
-        warnings.warn(
-            (
-                'xbacklight brightness method is deprecated and will be removed in the next release.'
-                ' Please use sysfiles or i2c or light or xrandr or ddcutil instead'
-            ),
-            DeprecationWarning
-        )
-
-    @classmethod
-    def set_brightness(cls, value: int, **kwargs):
-        '''
-        DEPRECATED.
-
-        Sets the screen brightness to a supplied value
-
-        Args:
-            value (int): the value to set the brightnes to
-            **kwargs: arbitrary keyword arguments. Ignored
-
-        Example:
-            ```python
-            import screen_brightness_control as sbc
-
-            # set the brightness to 100%
-            sbc.linux.XBacklight.set_brightness(100)
-            ```
-        '''
-        cls.__warn_deprecated()
-        subprocess.call([cls.executable, '-set', str(value)])
-
-    @classmethod
-    def get_brightness(cls, **kwargs) -> int:
-        '''
-        DEPRECATED.
-
-        Returns the screen brightness as reported by xbacklight
-
-        Returns:
-            int: from 0 to 100
-            **kwargs: arbitrary keyword arguments. Ignored
-
-        Raises:
-            ValueError: if xbacklight does not produce any valid output
-
-        Example:
-            ```python
-            import screen_brightness_control as sbc
-
-            current_brightness = sbc.linux.XBacklight.get_brightness()
-            ```
-        '''
-        cls.__warn_deprecated()
-        result = subprocess.check_output([cls.executable, '-get']).decode()
-        if not result:
-            raise ValueError('no valid output was received from xbacklight')
-        return int(round(float(str(result)), 0))
-
-
 class XRandr:
     '''collection of screen brightness related methods using the xrandr executable'''
 
@@ -789,7 +723,6 @@ class XRandr:
             return False
 
         xrandr_output = subprocess.check_output([cls.executable, '--verbose']).decode().split('\n')
-        connected_interfaces = tuple(cls.get_display_interfaces())
 
         valid_displays = []
         display_count = 0
@@ -799,7 +732,7 @@ class XRandr:
             if line == '':
                 continue
 
-            if line.startswith(connected_interfaces):
+            if not line.startswith((' ', '\t')) and 'connected' in line and 'disconnected' not in line:
                 if check_display(tmp_display):
                     valid_displays.append(tmp_display)
 
@@ -845,6 +778,7 @@ class XRandr:
     @classmethod
     def get_display_interfaces(cls) -> List[str]:
         '''
+        *DEPRECATED*  
         Returns the interfaces of each display, as reported by xrandr
 
         Returns:
@@ -858,6 +792,12 @@ class XRandr:
             # EG output: ['eDP-1', 'HDMI1', 'HDMI2']
             ```
         '''
+        warnings.warn(
+            (
+                'XRandr.get_display_interfaces is deprecated and will be removed in the next release.'
+                ' Please use XRandr.get_display_info instead'
+            ), DeprecationWarning
+        )
         out = subprocess.check_output([cls.executable, '-q']).decode().split('\n')
         return [i.split(' ')[0] for i in out if 'connected' in i and 'disconnected' not in i]
 
@@ -918,10 +858,6 @@ class XRandr:
         for i in info:
             subprocess.check_call([cls.executable, '--output', i['interface'], '--brightness', value])
 
-        # The get_brightness method takes the brightness value from get_display_info
-        # The problem is that that display info is cached, meaning that the brightness
-        # value is also cached. We must expire it here.
-
 
 class DDCUtil:
     '''collection of screen brightness related methods using the ddcutil executable'''
@@ -934,6 +870,18 @@ class DDCUtil:
     _max_brightness_cache: dict = {}
     '''Cache for monitors and their maximum brightness values'''
 
+    @staticmethod
+    def __call_ddcutil(command: list):
+        tries = 0
+        while True:
+            try:
+                return subprocess.check_output(command, stderr=subprocess.PIPE)
+            except subprocess.CalledProcessError:
+                if tries > 10:
+                    raise
+                tries += 1
+                time.sleep(0.04 if tries < 5 else 0.5)
+
     @classmethod
     def get_display_info(cls, display: Optional[Union[int, str]] = None) -> List[dict]:
         '''
@@ -974,11 +922,11 @@ class DDCUtil:
         valid_displays = __cache__.get('ddcutil_monitors_info')
         if valid_displays is None:
             raw_ddcutil_output = str(
-                subprocess.check_output(
+                cls.__call_ddcutil(
                     [
                         cls.executable, 'detect', '-v',
                         f'--sleep-multiplier={cls.sleep_multiplier}'
-                    ], stderr=subprocess.DEVNULL
+                    ]
                 )
             )[2:-1].split('\\n')
             # Use -v to get EDID string but this means output cannot be decoded.
@@ -1049,7 +997,8 @@ class DDCUtil:
 
             if check_display(tmp_display):
                 valid_displays.append(tmp_display)
-            __cache__.store('ddcutil_monitors_info', valid_displays)
+            if valid_displays:
+                __cache__.store('ddcutil_monitors_info', valid_displays)
 
         if display is not None:
             valid_displays = filter_monitors(display=display, haystack=valid_displays, include=['i2c_bus'])
@@ -1085,13 +1034,13 @@ class DDCUtil:
         for monitor in monitors:
             value = __cache__.get(f'ddcutil_brightness_{monitor["index"]}')
             if value is None:
-                cmd_out = subprocess.check_output(
+                cmd_out = cls.__call_ddcutil(
                     [
                         cls.executable,
                         'getvcp', '10', '-t',
                         '-b', str(monitor['bus_number']),
                         f'--sleep-multiplier={cls.sleep_multiplier}'
-                    ], stderr=subprocess.DEVNULL
+                    ]
                 ).decode().split(' ')
 
                 value = int(cmd_out[-2])
@@ -1147,12 +1096,12 @@ class DDCUtil:
             if cls._max_brightness_cache[cache_ident] != 100:
                 value = int((value / 100) * cls._max_brightness_cache[cache_ident])
 
-            subprocess.check_call(
+            cls.__call_ddcutil(
                 [
                     cls.executable, 'setvcp', '10', str(value),
                     '-b', str(monitor['bus_number']),
                     f'--sleep-multiplier={cls.sleep_multiplier}'
-                ], stderr=subprocess.DEVNULL
+                ]
             )
 
 
